unit UFormMain;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Menus, ActnList, ComCtrls, UPersistentForm, ULastOpenedList, UGrammer,
  XMLConf;

type

  { TFormMain }

  TFormMain = class(TForm)
    ABuildCompiler: TAction;
    ALookupTables: TAction;
    AToolbarVisible: TAction;
    AProjectOpenRecent: TAction;
    AProjectClose: TAction;
    AGrammerSettings: TAction;
    AProjectNew: TAction;
    AProjectSave: TAction;
    AExit: TAction;
    AProjectOpen: TAction;
    AProjectSaveAs: TAction;
    ARules: TAction;
    ActionList1: TActionList;
    CoolBar1: TCoolBar;
    LastOpenedList1: TLastOpenedList;
    MainMenu1: TMainMenu;
    MemoEbnf: TMemo;
    MemoBnf: TMemo;
    MenuItem1: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem12: TMenuItem;
    MenuItem13: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItemRecent: TMenuItem;
    MenuItemToolbar: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    OpenDialog1: TOpenDialog;
    PageControl1: TPageControl;
    PopupMenuOpenRecent: TPopupMenu;
    SaveDialog1: TSaveDialog;
    TabSheetRules: TTabSheet;
    TabSheetBnf: TTabSheet;
    TabSheetEbnf: TTabSheet;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    procedure ABuildCompilerExecute(Sender: TObject);
    procedure ALookupTablesExecute(Sender: TObject);
    procedure AToolbarVisibleExecute(Sender: TObject);
    procedure AExitExecute(Sender: TObject);
    procedure AGrammerSettingsExecute(Sender: TObject);
    procedure ARulesExecute(Sender: TObject);
    procedure AProjectNewExecute(Sender: TObject);
    procedure AProjectOpenExecute(Sender: TObject);
    procedure AProjectOpenRecentExecute(Sender: TObject);
    procedure AProjectCloseExecute(Sender: TObject);
    procedure AProjectSaveAsExecute(Sender: TObject);
    procedure AProjectSaveExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormShow(Sender: TObject);
    procedure LastOpenedList1Change(Sender: TObject);
  private
    procedure GrammerChange(Sender: TObject);
    procedure GrammerDestroy(Sender: TObject);
  public
    procedure ReloadMemo;
    procedure UpdateInterface;
    procedure ProjectOpen(FileName: string);
    procedure ProjectNew;
    procedure LoadConfig(XmlConfig: TXmlConfig);
    procedure SaveConfig(XmlConfig: TXmlConfig);
  end;

var
  FormMain: TFormMain;


implementation

{$R *.lfm}

uses
  UFormRules, UCore, UFormGrammer, UBuilder, UFormLookupTables;

resourcestring
  SUnsavedChanges = 'Unsaved changes';
  SDoYouWantToSaveProject = 'There are unsaved changes in project. Do you want to save them?';
  SNewGrammer = 'New grammer.grm';

  { TFormMain }

procedure TFormMain.ARulesExecute(Sender: TObject);
var
  FormRules: TFormRules;
begin
  FormRules := TFormRules.Create(nil);
  FormRules.Rules := Core.Grammer.Rules;
  FormRules.ShowModal;
  ReloadMemo;
  UpdateInterface;
  FreeAndNil(FormRules);
end;

procedure TFormMain.AProjectSaveAsExecute(Sender: TObject);
begin
  SaveDialog1.InitialDir := ExtractFileDir(Core.Grammer.FileName);
  SaveDialog1.FileName := ExtractFileName(Core.Grammer.FileName);
  if SaveDialog1.Execute then begin
    LastOpenedList1.AddItem(SaveDialog1.FileName);
    Core.Grammer.SaveToXmlFile(SaveDialog1.FileName);
    UpdateInterface;
  end;
end;

procedure TFormMain.AProjectSaveExecute(Sender: TObject);
begin
  Core.Grammer.SaveToXmlFile(Core.Grammer.FileName);
  UpdateInterface;
end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Core.PersistentForm1.Save(Self);
  Core.Finalize;
end;

procedure TFormMain.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  AProjectClose.Execute;
  if Assigned(Core.Grammer) then CanClose := False;
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  Core.PersistentForm1.Load(Self, True);
  Core.Initialize;
  PageControl1.TabIndex := 0;
  ReloadMemo;
  UpdateInterface;

  FormRules.Rules := Core.Grammer.Rules;
  FormRules.ManualDock(TabSheetRules, nil, alClient);
  FormRules.Align := alClient;
  FormRules.Show;
end;

procedure TFormMain.LastOpenedList1Change(Sender: TObject);
begin
  LastOpenedList1.LoadToMenuItem(MenuItemRecent, AProjectOpenRecentExecute);
  LastOpenedList1.LoadToMenuItem(PopupMenuOpenRecent.Items, AProjectOpenRecentExecute);
end;

procedure TFormMain.ReloadMemo;
begin
  if Assigned(Core.Grammer) then begin
    MemoBnf.Lines.Text := Core.Grammer.ExportAsString(gnBnf);
    MemoEbnf.Lines.Text := Core.Grammer.ExportAsString(gnEbnf);
  end else begin
    MemoBnf.Lines.Clear;
    MemoEbnf.Lines.Clear;
  end;
end;

procedure TFormMain.UpdateInterface;
var
  NewTitle: string;
begin
  NewTitle := Application.Title;
  if Assigned(Core.Grammer) then begin
    NewTitle := ExtractFileNameWithoutExt(ExtractFileName(Core.Grammer.FileName)) + ' - ' + NewTitle;
    if Core.Grammer.Modified then NewTitle := NewTitle + '*';
  end;
  Caption := NewTitle;
  AProjectClose.Enabled := Assigned(Core.Grammer);
  AProjectSave.Enabled := Assigned(Core.Grammer) and Core.Grammer.Modified;
  AProjectSaveAs.Enabled := Assigned(Core.Grammer);
  AGrammerSettings.Enabled := Assigned(Core.Grammer);
  ABuildCompiler.Enabled := Assigned(Core.Grammer);
  ARules.Enabled := Assigned(Core.Grammer);
  CoolBar1.Visible := AToolbarVisible.Checked;
end;

procedure TFormMain.GrammerChange(Sender: TObject);
begin
  ReloadMemo;
  UpdateInterface;
  if Assigned(FormRules) then begin
    FormRules.Rules := Core.Grammer.Rules;
    FormRules.ReloadList;
    FormRules.UpdateInterface;
  end;
end;

procedure TFormMain.GrammerDestroy(Sender: TObject);
begin
  ReloadMemo;
  UpdateInterface;
  if Assigned(FormRules) then begin
    FormRules.Rules := nil;
    FormRules.ReloadList;
    FormRules.UpdateInterface;
  end;
end;

procedure TFormMain.ProjectOpen(FileName: string);
begin
  AProjectClose.Execute;
  if not Assigned(Core.Grammer) then begin
    LastOpenedList1.AddItem(FileName);
    ProjectNew;
    Core.Grammer := TGrammer.Create;
    Core.Grammer.OnChange := GrammerChange;
    Core.Grammer.OnDestroy := GrammerDestroy;
    Core.Grammer.LoadFromXmlFile(FileName);
    Core.Grammer.Change;
  end;
end;

procedure TFormMain.ProjectNew;
begin
  if Assigned(Core.Grammer) then FreeAndNil(Core.Grammer);
  Core.Grammer := TGrammer.Create;
  Core.Grammer.OnChange := GrammerChange;
  Core.Grammer.OnDestroy := GrammerDestroy;
  Core.Grammer.FileName := SNewGrammer;
  Core.Grammer.Change;
end;

procedure TFormMain.LoadConfig(XmlConfig: TXmlConfig);
begin
  AToolbarVisible.Checked := XmlConfig.GetValue('ToolbarVisible', True);
  LastOpenedList1.LoadFromXMLConfig(XMLConfig, 'LastOpenedFiles');
end;

procedure TFormMain.SaveConfig(XmlConfig: TXmlConfig);
begin
  XmlConfig.SetValue('ToolbarVisible', AToolbarVisible.Checked);
  LastOpenedList1.SaveToXMLConfig(XMLConfig, 'LastOpenedFiles');
end;

procedure TFormMain.AExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.AGrammerSettingsExecute(Sender: TObject);
var
  FormGrammer: TFormGrammer;
begin
  FormGrammer := TFormGrammer.Create(nil);
  FormGrammer.LoadData(Core.Grammer);
  if FormGrammer.ShowModal = mrOk then begin
    FormGrammer.SaveData(Core.Grammer);
    UpdateInterface;
  end;
  FreeAndNil(FormGrammer);
end;

procedure TFormMain.AProjectOpenRecentExecute(Sender: TObject);
begin
  if Sender is TMenuItem then begin
    AProjectClose.Execute;
    if not Assigned(Core.Grammer) then begin
      ProjectOpen(TMenuItem(Sender).Caption);
    end;
  end;
end;

procedure TFormMain.AProjectNewExecute(Sender: TObject);
begin
  AProjectClose.Execute;
  if not Assigned(Core.Grammer) then begin
    ProjectNew;
  end;
end;

procedure TFormMain.ABuildCompilerExecute(Sender: TObject);
var
  Builder: TBuilder;
begin
  Builder := TBuilder.Create;
  try
    Builder.Grammer := Core.Grammer;
    Builder.BuildCompiler;
  finally
    FreeAndNil(Builder);
  end;
end;

procedure TFormMain.ALookupTablesExecute(Sender: TObject);
var
  FormLookupTables: TFormLookupTables;
begin
  FormLookupTables := TFormLookupTables.Create(nil);
  try
    FormLookupTables.LookupTables := Core.Grammer.LookupTables;
    if FormLookupTables.ShowModal = mrOk then begin

    end;
  finally
    FreeAndNil(FormLookupTables);
  end;
end;

procedure TFormMain.AToolbarVisibleExecute(Sender: TObject);
begin
  UpdateInterface;
end;

procedure TFormMain.AProjectOpenExecute(Sender: TObject);
begin
  OpenDialog1.InitialDir := ExtractFileDir(LastOpenedList1.GetFirstFileName);
  OpenDialog1.FileName := ExtractFileName(LastOpenedList1.GetFirstFileName);
  if OpenDialog1.Execute then begin
    ProjectOpen(OpenDialog1.FileName);
  end;
end;

procedure TFormMain.AProjectCloseExecute(Sender: TObject);
var
  DialogResult: TModalResult;
begin
  if not Assigned(Core.Grammer) then Exit;

  DialogResult := mrNo;
  if Core.Grammer.Modified then begin
    DialogResult := MessageDlg(SUnsavedChanges, SDoYouWantToSaveProject, mtConfirmation, [mbYes, mbNo, mbCancel], 0);
    if DialogResult = mrYes then begin
      AProjectSave.Execute;
    end;
  end;
  if (DialogResult = mrNo) or (DialogResult = mrYes) then begin
    FreeAndNil(Core.Grammer);
  end;
end;

end.

